// AUTO-GENERATED - DO NOT EDIT. See src/common/tools/gen_listings_and_webworkers.ts.

import { g } from '../../../../../../shader/execution/expression/call/builtin/pack4xU8Clamp.spec.js';
import { wrapTestGroupForWorker } from '../../../../../../../common/runtime/helper/wrap_for_worker.js';

wrapTestGroupForWorker(g);
