@charset
"iso-8859-15";
span {
	background-color: #00FF00;
	font-weight: bold;
	color: #00FF00;
	font-family: Arial, Helvetica, sans-serif;
	}
.ýäè {
	background-color: #FF0000;
	font-weight: bold;
	color: #FF0000;
	font-family: Arial, Helvetica, sans-serif;
	}
